#!/usr/bin/env perl
#
# Converts journal names from full form to abbreviated (either with dots or without dots).
# Creates two new .bib files, so original .bib file is unchanged.
# Abbreviation list is file of multiple rows, one row per journal.
# Each row is of the format 
#	<Full name> = <Abbrev.>, where abbreviation uses dots "." for abbreviated words,
# or of the format
#	<Full name> = <Abbrev> = <Abbrev.>, where only second abbreviation uses dots "." for abbreviated words.
# Empty lines in journal list are ignored
# Comment lines start with route '#'
# Appart from that, this is a simple find and replace script, no formating done.
#
# Notes: 
#	Will leave journal names within double curly brackets untouched!
#	Will try to match journal name with additional 'the' and commas ',' at the beginning and end
#	Will try to match journal name with additional (or less) commas between words
#	Will try to match unicode accentual chars (ü,ö,ä,é,á,í,ú,ó,è,à,ì,ù,ò,û) in full or abbreviated names with LaTeX escaped chars (\"u, {\"u} and so on) in BibTeX journal fields.
# 	Full name (search term) is case insensitive, but abbreviation case is taken as given in abbreviations list
# 	Excess white spaces, as well as leading and trailing white space are ignored.
#	Journal names can include the characters ()[]-+&:?
#		Please do not attempt to escape any character the journal names.
#
# The script takes two arguments, the .bib file to be transformed and the journal-abbreviation list.
# Computational complexity: <BibTeX entry count> x <Journal abbreviation count>
# Written by Stilianos Louca, May 09, 2013
# This script comes with no warranty. Use at your own risk.
#
#------------------------------------------------------------






sub prepare_needle{
	$needle = $_[0];
		
	# prepare for possibility that some characters in .bib fields are not escaped yet, 
	# so be tollerant towards that in your search
	# We assume that journal name list does NOT escape anything
	$needle =~ s/&/\\\\\*&/g;
	
	# Take care of accented characters
	$needle =~ s/ü/(\\\{\\\\\\\"u\\\}|\\\\\\\"u|ü)/g;
	$needle =~ s/ö/(\\\{\\\\\\\"o\\\}|\\\\\\\"o|ö)/g;
	$needle =~ s/ä/(\\\{\\\\\\\"a\\\}|\\\\\\\"a|ä)/g;
	$needle =~ s/é/(\\\{\\\\\\\'e\\\}|\\\\\\\'e|é)/g;
	$needle =~ s/á/(\\\{\\\\\\\'a\\\}|\\\\\\\'a|á)/g;
	$needle =~ s/í/(\\\{\\\\\\\'i\\\}|\\\\\\\'i|í)/g;
	$needle =~ s/ú/(\\\{\\\\\\\'u\\\}|\\\\\\\'u|ú)/g;
	$needle =~ s/ó/(\\\{\\\\\\\'o\\\}|\\\\\\\'o|ó)/g;
	$needle =~ s/è/(\\\{\\\\\\\`e\\\}|\\\\\\\`e|é)/g;
	$needle =~ s/à/(\\\{\\\\\\\`a\\\}|\\\\\\\`a|à)/g;
	$needle =~ s/ì/(\\\{\\\\\\\`i\\\}|\\\\\\\`i|ì)/g;
	$needle =~ s/ù/(\\\{\\\\\\\`u\\\}|\\\\\\\`u|ù)/g;
	$needle =~ s/ò/(\\\{\\\\\\\`o\\\}|\\\\\\\`o|ò)/g;
	$needle =~ s/û/(\\\{\\\\\\\^u\\\}|\\\\\\\^u|û)/g;
	
	$needle =~ s/Ü/(\\\{\\\\\\\"U\\\}|\\\\\\\"U|Ü)/g;
	$needle =~ s/Ö/(\\\{\\\\\\\"O\\\}|\\\\\\\"O|Ö)/g;
	$needle =~ s/Ä/(\\\{\\\\\\\"A\\\}|\\\\\\\"A|Ä)/g;
	$needle =~ s/É/(\\\{\\\\\\\'E\\\}|\\\\\\\'E|É)/g;
	$needle =~ s/Á/(\\\{\\\\\\\'A\\\}|\\\\\\\'A|É)/g;
	$needle =~ s/Í/(\\\{\\\\\\\'I\\\}|\\\\\\\'I|Í)/g;
	$needle =~ s/Ú/(\\\{\\\\\\\'U\\\}|\\\\\\\'U|Ú)/g;
	$needle =~ s/Ó/(\\\{\\\\\\\'O\\\}|\\\\\\\'O|Ó)/g;
	$needle =~ s/È/(\\\{\\\\\\\`E\\\}|\\\\\\\`E|É)/g;
	$needle =~ s/À/(\\\{\\\\\\\`A\\\}|\\\\\\\`A|À)/g;
	$needle =~ s/Ì/(\\\{\\\\\\\`I\\\}|\\\\\\\`I|Ì)/g;
	$needle =~ s/Ù/(\\\{\\\\\\\`U\\\}|\\\\\\\`U|Ù)/g;
	$needle =~ s/Ò/(\\\{\\\\\\\`O\\\}|\\\\\\\`O|Ò)/g;
	$needle =~ s/Û/(\\\{\\\\\\\^U\\\}|\\\\\\\^U|Û)/g;

	return $needle;
}


sub prepare_subst{
	$subst = $_[0];
	
	$subst =~ s/&/\\&/g;
	
	$subst =~ s/ü/\\\"u/g;
	$subst =~ s/ö/\\\"o/g;
	$subst =~ s/ä/\\\"a/g;
	$subst =~ s/é/\\\'e/g;
	$subst =~ s/á/\\\'a/g;
	$subst =~ s/í/\\\'i/g;
	$subst =~ s/ú/\\\'u/g;
	$subst =~ s/ó/\\\'o/g;
	$subst =~ s/è/\\\`e/g;
	$subst =~ s/à/\\\`a/g;
	$subst =~ s/ì/\\\`i/g;
	$subst =~ s/ù/\\\`u/g;
	$subst =~ s/ò/\\\`o/g;
	$subst =~ s/û/\\\^u/g;
	
	$subst =~ s/Ü/\\\"U/g;
	$subst =~ s/Ö/\\\"O/g;
	$subst =~ s/Ä/\\\"A/g;
	$subst =~ s/É/\\\'E/g;
	$subst =~ s/Á/\\\'A/g;
	$subst =~ s/Í/\\\'I/g;
	$subst =~ s/Ú/\\\'U/g;
	$subst =~ s/Ó/\\\'O/g;
	$subst =~ s/È/\\\`E/g;
	$subst =~ s/À/\\\`A/g;
	$subst =~ s/Ì/\\\`I/g;
	$subst =~ s/Ù/\\\`U/g;
	$subst =~ s/Ò/\\\`O/g;
	$subst =~ s/Û/\\\^U/g;
	
	return $subst;
}



if($#ARGV < 1){
	die "Missing input files: <BibTeX Database> and <Journal Abbreviation List>\n";
}

if(!($ARGV[0] =~ /.bib$/)){
	die "Input file does not seem to be a BibTeX file (missing extension .bib)\n";
}


# Get the list of journal names in a variable
open(JOURNALS,  $ARGV[1]) or die "Can't open ".$ARGV[1].": $!";
@journalnames = <JOURNALS>;
close(JOURNALS);

# Open the bib file in a variable
open(BIBFILE,  $ARGV[0]) or die "Can't open ".$ARGV[0].": $!";
@file_dots = <BIBFILE>;
@file_nodots = @file_dots;
close(BIBFILE);

$logFile = $ARGV[0];
if(!($logFile =~ s/.bib$/_abbr_log/)){
	$logFile .= "_abbr_log";
}
open (LOGFILE, '> '.$logFile) or die "Can't open $logFile: $!";
$replacementCounts = 0;

# For each element of the list of journal names we extract the full name of the journal and the abbreviated name
foreach $journame (@journalnames) {
	#remove white space at beginning and end
    $journame =~ s/^\s+|\s+$//g;
    
    if(($journame eq "") || ($journame =~ /^#/)){
    	# is just a comment line or empty line
    	next;
    }
    
	# remove white space around '=' signs
	$journame =~ s/\s*=\s*/=/g;
	
	# remove excess white space
	$journame =~ s/\h+/ /g;
	
	# split based on "="
	if($journame =~ /^(.*)=(.*)=(.*)$/){ # has three journal versions
		$full = $1;
		$abbrvNoDots = $2;
		$abbrvDots = $3;
	}elsif($journame =~ /^(.*)=(.*)$/){ # has two journal versions
		$full = $1;
		$abbrvDots = $2;
		$abbrvNoDots = $abbrvDots;
		$abbrvNoDots =~ s/\.//g;
	}else{
		next;
	}
	
	
	$full_needle = $full;
	$abbrvDots_needle = $abbrvDots;
	$abbrvNoDots_needle = $abbrvNoDots;
	
	# Escape certain chars that would be otherwise influencing the final regex
	$full_needle =~ s/\(/\\\(/g;
	$full_needle =~ s/\)/\\\)/g;
	$full_needle =~ s/\[/\\\[/g;
	$full_needle =~ s/\]/\\\]/g;
	$full_needle =~ s/\-/\\\-/g;
	$full_needle =~ s/\+/\\\+/g;
	$full_needle =~ s/\:/\\\:/g;
	$full_needle =~ s/\?/\\\?/g;

	$abbrvDots_needle =~ s/\(/\\\(/g;
	$abbrvDots_needle =~ s/\)/\\\)/g;
	$abbrvDots_needle =~ s/\[/\\\[/g;
	$abbrvDots_needle =~ s/\]/\\\]/g;
	$abbrvDots_needle =~ s/\-/\\\-/g;
	$abbrvDots_needle =~ s/\+/\\\+/g;
	$abbrvDots_needle =~ s/\:/\\\:/g;
	$abbrvDots_needle =~ s/\?/\\\?/g;
	
	$abbrvNoDots_needle =~ s/\(/\\\(/g;
	$abbrvNoDots_needle =~ s/\)/\\\)/g;
	$abbrvNoDots_needle =~ s/\[/\\\[/g;
	$abbrvNoDots_needle =~ s/\]/\\\]/g;
	$abbrvNoDots_needle =~ s/\-/\\\-/g;
	$abbrvNoDots_needle =~ s/\+/\\\+/g;
	$abbrvNoDots_needle =~ s/\:/\\\:/g;
	$abbrvNoDots_needle =~ s/\?/\\\?/g;
	
	
	# make matching white-space tollerant (leading, trailing and excess middle white space)
	# also some full-name entries might be missing or have to many commas
	# so modify search terms (needles) to include some regex code for white-space + comma flexibility
	$full_needle =~ s/^[,\ ]*(?:the)?[,\ ]*|[,\ ]*(?:the)?[,\ ]*$//gi;	# exclude beginning and end of file for next step
	$full_needle =~ s/(?:\h+|\ *,+\ *)/\(?:\\h+\|\\ *,+\\ *\)/gi;
	# Include possibility of some "forgotten" parts at beginning and end, like 'the'
	$full_needle =~ s/\h*,*(?:the)?\h*,*\h*$/\\h*,*\\h*\(?:the\)?\\h*,*\\h*/gi;
	$full_needle =~ s/^\h*,*(?:the)?\h*,*\h*/\\h*,*\\h*\(?:the\)?\\h*,*\\h*/gi;

	
	$abbrvDots_needle 	=~ s/ /\\h+/g;
	$abbrvDots_needle 	= '\h*,*\h*'.$abbrvDots_needle.'\h*,*\h*';
	
	$abbrvNoDots_needle =~ s/ /\\h+/g;
	$abbrvNoDots_needle = '\h*,*\h*'.$abbrvNoDots_needle.'\h*,*\h*';
	
	# some more needle adjustments
	$full_needle 		= prepare_needle($full_needle);
	$abbrvDots_needle 	= prepare_needle($abbrvDots_needle);
	$abbrvNoDots_needle	= prepare_needle($abbrvNoDots_needle);
	
	
	#Escape some chars in replacement names before putting them into the BibTeX file
	$full 			= prepare_subst($full);
	$abbrvDots		= prepare_subst($abbrvDots);
	$abbrvNoDots 	= prepare_subst($abbrvNoDots);
		
	# replace journals with dotted abbreviations
	foreach $line (@file_dots){
		if($line =~ /journal\s*=/i){
				# case where journal name is in curly brakets
			if(!($line =~ s/=\s*\{($full_needle|$abbrvNoDots_needle)\}/= \{$abbrvDots\}/gi)){
				# case where journal name is in quotes (also valid BibTeX syntax)
				if($line =~ s/=\s*\"($full_needle|$abbrvNoDots_needle)\"/= \"$abbrvDots\"/gi){
					print LOGFILE $1.' = '.$abbrvDots."\n";
					++$replacementCounts;
				}
			}else{
				print LOGFILE $1.' = '.$abbrvDots."\n";
				++$replacementCounts;
			}
		}
	}
	
	# replace journals with non-dotted abbreviations
	foreach $line (@file_nodots){
		if($line =~ /journal\s*=/i){
				# case where journal name is in curly brakets
			if(!($line =~ s/=\s*\{(?:$full_needle|$abbrvDots_needle)\}/= \{$abbrvNoDots\}/gi)){
				# case where journal name is in quotes (also valid BibTeX syntax)
				$line =~ s/=\s*\"(?:$full_needle|$abbrvDots_needle)\"/= \"$abbrvNoDots\"/gi;
			}
		}
	}
}
print LOGFILE "\nReplaced $replacementCounts journals in BibTeX file ".$ARGV[0]."\n";
close (LOGFILE);

# Print to stdout, can be redirected to a file with " > somefile.bib "
$outFile = $ARGV[0];
if(!($outFile =~ s/.bib$/_abbr_dots.bib/)){
	$outFile .= "_abbr_dots.bib";
}
open (OUTFILE, '> '.$outFile) or die "Can't open $outFile: $!";
print OUTFILE @file_dots;
close (OUTFILE);

$outFile = $ARGV[0];
if(!($outFile =~ s/.bib$/_abbr_nodots.bib/)){
	$outFile .= "_abbr_nodots.bib";
}
open (OUTFILE, '> '.$outFile) or die "Can't open $outFile: $!";
print OUTFILE @file_nodots;
close (OUTFILE);